@@ -1,17 +1,28 @@
 package Module::Collect;
+use 5.006_002;
 use strict;
 use warnings;
-our $VERSION = '0.05';
+our $VERSION = '0.06';
+
+use Text::Glob qw(glob_to_regex);
 
 use Carp;
-use File::Find::Rule;
+use File::Find;
+use File::Spec;
 use Module::Collect::Package;
 
 sub new {
     my($class, %args) = @_;
 
-    $args{modules}  = [];
-    $args{pattern} = '*.pm' unless $args{pattern};
+    $args{modules}   = [];
+    $args{pattern} ||= '*.pm';
+
+    if($args{prefix}){
+        $args{prefix} .= '::';
+    }
+    else{
+        $args{prefix} = '';
+    }
 
     my $self = bless { %args }, $class;
     $self->_find_modules;
@@ -22,20 +33,25 @@ sub new {
 sub _find_modules {
     my $self = shift;
 
-    my $path = $self->{path} || [];
-       $path = [ $path ] unless ref($path) eq 'ARRAY';
+    my $path = $self->{path} || return;
 
-    for my $dirpath (@{ $path }) {
-        next unless -d $dirpath;
+    my $pattern = glob_to_regex($self->{pattern});
 
-        my $rule = File::Find::Rule->new;
-        $rule->file;
-        $rule->name($self->{pattern});
+    my @modules;
 
-        my @modules = $rule->in($dirpath);
-        for my $modulefile (@modules) {
-            $self->_add_module($modulefile);
-        }
+    my $wanted  = sub{
+        return unless -f;
+        return unless /$pattern/;
+        push @modules, File::Spec->canonpath($File::Find::name);
+    };
+
+    for my $dir(ref($path) eq 'ARRAY' ? @{$path} : $path){
+        next unless -d $dir;
+        find($wanted, $dir);
+    }
+
+    for my $modulefile(@modules){
+        $self->_add_module($modulefile);
     }
 }
 
@@ -56,38 +72,23 @@ sub _extract_package {
 
     open my $fh, '<', $modulefile or croak "$modulefile: $!";
     my $prefix = $self->{prefix};
-    $prefix .= '::' if $prefix;
-    $prefix = '' unless $prefix;
 
-    return _extract_multiple_package($fh, $prefix) if $self->{multiple};
-    my $in_pod = 0;
-    while (<$fh>) {
-        $in_pod = 1 if m/^=\w/;
-        $in_pod = 0 if /^=cut/;
-        next if ($in_pod || /^=cut/);  # skip pod text
-        next if /^\s*\#/;
-
-        /^\s*package\s+($prefix.*?)\s*;/ and return $1;
-    }
-    return;
-}
-
-sub _extract_multiple_package {
-    my($fh, $prefix) = @_;
-
-    my $in_pod = 0;
+    my $multiple = $self->{multiple};
     my @packages;
-    while (<$fh>) {
-        $in_pod = 1 if m/^=\w/;
-        $in_pod = 0 if /^=cut/;
-        next if ($in_pod || /^=cut/);  # skip pod text
-        next if /^\s*\#/;
 
-        /^\s*package\s+($prefix.*?)\s*;/ and push @packages, $1;
+    while (<$fh>) {
+        next if /\A =\w/xms .. /\A =cut \b/xms; # skip pod sections
+
+        if(/\A \s* package \s+ ($prefix \S*) \s* ;/xms){
+            push @packages, $1;
+
+            last unless $multiple;
+        }
     }
     return @packages;
 }
 
+
 sub modules {
     my $self = shift;
     $self->{modules};
@@ -100,16 +101,16 @@ __END__
 
 =head1 NAME
 
-Module::Collect - module files are collected from some directories
+Module::Collect - Collect sub-modules in directories
 
 =head1 SYNOPSIS
 
   use Module::Collect;
   my $collect = Module::Collect->new(
-      path   => '/foo/bar/plugins',
-      prefix => 'MyApp::Plugin', # not required option
-      pattern => '*.pm',         # not required option
-      multiple => 1,             # not required option see t/06_multiple.t
+      path     => '/foo/bar/plugins',
+      prefix   => 'MyApp::Plugin', # optional
+      pattern  => '*.pm',          # optional
+      multiple => 1,               # optional, see t/06_multiple.t
   );
 
   my @modules = @{ $collect->modules };
@@ -122,6 +123,8 @@ Module::Collect - module files are collected from some directories
 
 =head1 DESCRIPTION
 
+Module::Collect finds sub-modules, or plugins in directories.
+
 The following directory composition
 
   $ ls -R t/plugins
@@ -155,6 +158,12 @@ results
 
 Kazuhiro Osawa E<lt>ko@yappo.ne.jpE<gt>
 
+lopnor
+
+gfx
+
+mattn
+
 =head1 INSPIRED BY
 
 L<Plagger>, L<Module::Pluggable>
@@ -165,9 +174,9 @@ L<Module::Collect::Package>
 
 =head1 REPOSITORY
 
-  svn co http://svn.coderepos.org/share/lang/perl/Module-Collect/trunk Module-Collect
+  git clone git://github.com/yappo/p5-Module-Collect.git
 
-Module::Collect is Subversion repository is hosted at L<http://coderepos.org/share/>.
+Module::Collect is git repository is hosted at L<http://github.com/yappo/p5-Module-Collect>.
 patches and collaborators are welcome.
 
 =head1 LICENSE
